# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013 Canonical
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

"""Tests for managing locations in Weather app"""

from __future__ import absolute_import

from testtools.matchers import Equals
from autopilot.matchers import Eventually

from ubuntu_weather_app.tests import WeatherTestCase, DatabaseMixin
from ubuntu_weather_app.tests.weatherdata import locations_data

class TestLocationManager(WeatherTestCase, DatabaseMixin):
    def setUp(self):
        self.clean_db()
        super(TestLocationManager, self).setUp()
        self.assertThat(
            self.main_window.get_qml_view().visible, Eventually(Equals(True)))

    def _open_add_location_page(self):
        """Opens the AddLocationPage"""
        # click on listitem to open AddLocationPage
        locationList = self.main_window.get_object('QQuickListView', 'LocationList')
        addCityItem = locationList.get_children()[0]
        self.pointing_device.move_to_object(addCityItem)
        self.pointing_device.click()

        addLocPage = self.main_window.get_object("AddLocationPage", "AddLocationPage")
        self.assertThat(addLocPage.visible, Eventually(Equals(True)))

    def test_add_location(self):
        """Adds a location"""
        self._open_add_location_page()

        # insert city name to search for
        searchField = self.main_window.get_object("TextField", "SearchField")
        self.pointing_device.move_to_object(searchField)
        self.pointing_device.click()
        self.keyboard.type("London")
        self.assertThat(searchField.text, Eventually(Equals("London")))
        self.keyboard.press_and_release('Enter')

        # wait for results and click on the first item
        resultsList = self.main_window.get_object('QQuickListView', 'SearchResultList')
        self.assertThat(resultsList.visible, Eventually(Equals(True), timeout=30))
        firstResult = resultsList.get_children()[0].get_children()[0]
        self.pointing_device.move_to_object(firstResult)
        self.pointing_device.click()

        # LocationManagerPage should be visible and "London" added
        locationList = self.main_window.get_object('QQuickListView', 'LocationList')
        addedItem = locationList.get_children()[0]
        self.assertThat(addedItem.get_children()[1].text, Eventually(Equals("London")))

        # a location is defined, so toolbar should be visible
        toolbar = self.main_window.get_toolbar()
        self.assertThat(toolbar.opened, Eventually(Equals(True)))

    def test_toolbar_hidden_when_no_location_is_defined(self):
        """Toolbar is not visible when there's no location is defined"""
        toolbar = self.main_window.get_toolbar()
        self.assertThat(toolbar.opened, Eventually(Equals(False)))

    def test_search_city(self):
        """Search for cities"""
        self._open_add_location_page()

        # insert city name to search for
        searchField = self.main_window.get_object("TextField", "SearchField")
        self.pointing_device.move_to_object(searchField)
        self.pointing_device.click()
        self.keyboard.type("London")
        self.assertThat(searchField.text, Eventually(Equals("London")))
        self.keyboard.press_and_release('Enter')

        # wait for results and click on the first item
        resultsList = self.main_window.get_object('QQuickListView', 'SearchResultList')
        self.assertThat(resultsList.visible, Eventually(Equals(True), timeout=30))
        firstResult = resultsList.get_children()[0].get_children()[0]
        self.assertThat(firstResult.text, Eventually(Equals("London, GB")))

        # clear search field and do another search
        for x in range(6):
            self.keyboard.press_and_release('BackSpace')
        self.assertThat(searchField.text, Eventually(Equals("")))
        self.keyboard.type("Hanbu")
        for x in range(3):
            self.keyboard.press_and_release('BackSpace')
        self.keyboard.type("mburg")
        self.keyboard.press_and_release('Enter')
        firstResult = resultsList.get_children()[0].get_children()[0]
        self.assertThat(firstResult.text, Eventually(Equals("Hamburg, DE")))

    def test_cancel_adding_location(self):
        """Cancel the cities search"""
        self._open_add_location_page()
        self.main_window.click_toolbar_button("BackButton")
        locationList = self.main_window.get_object('QQuickListView', 'LocationList')
        self.assertThat(locationList.visible, Eventually(Equals(True)))

class TestLocationManagerWithLocation(WeatherTestCase, DatabaseMixin):
    def setUp(self):
        self.clean_db()
        self.launch_and_quit_app()
        # add one location to storage
        self.save_locations_to_storage([locations_data[0]])
        super(TestLocationManagerWithLocation, self).setUp()
        self.assertThat(
            self.main_window.get_qml_view().visible, Eventually(Equals(True)))

    def _open_location_manager(self):
        """Opens the location manager"""
        self.main_window.open_toolbar()
        self.assertThat(self.main_window.get_toolbar().opened, Eventually(Equals(True)))
        self.main_window.click_toolbar_button("EditButton")

    def test_remove_location(self):
        """Removes location"""
        # check loading screen diappears
        loadingPage = self.main_window.get_object("Tabs", "LoadingTabs")
        self.assertThat(loadingPage.visible, Eventually(Equals(False)))

        # go to the location manegr
        self._open_location_manager()

        # swipe right to delete the first location
        locationList = self.main_window.get_object('QQuickListView', 'LocationList')
        locItem = locationList.get_children()[0]
        loc_x, loc_y, loc_w, loc_h = locItem.globalRect
        self.pointing_device.move(loc_x + 10, loc_y + loc_h / 2)
        self.pointing_device.press()
        self.pointing_device.move(loc_x + loc_w - 10, loc_y + loc_h / 2)
        self.pointing_device.release()

        # toolbar disappears since no locations defined
        self.assertThat(self.main_window.get_toolbar().opened, Eventually(Equals(False)))

    def test_toolbar_opened_when_location_is_defined(self):
        """Toolbar is visible since there's a location defined"""
        self._open_location_manager()
        toolbar = self.main_window.get_toolbar()
        self.assertThat(toolbar.opened, Eventually(Equals(True)))
